<?php
// $Id$

/**
 * @file
 * Helper functions for communicating with the SCORM Cloud API.
 */
 
/**
 * Create an api engine for calling the api.
 */
function scorm_cloud_api_service_engine(){
  $path = scorm_cloud_api_library_path();
  if ($path && file_exists($path . '/ScormEngineService.php')) {
    require_once('./'. $path . '/ScormEngineService.php');
    $appid = variable_get('scorm_cloud_appid', '');  
    $secretkey = variable_get('scorm_cloud_secretkey', ''); 
    $engine_url = variable_get('scorm_cloud_servicesurl', 'http://cloud.scorm.com/EngineWebServices');
  
    return new ScormEngineService($engine_url, $appid, $secretkey, SCORM_CLOUD_ORIGIN);
 }
 else {
  drupal_set_message(t('You must install the SCORM Cloud PHP libary. Please see the README.txt file in the SCORM Cloud module directory.'), 'error', FALSE);
 }
}
/**
 * Determine the path to the SCORM Cloud php library.
 */
function scorm_cloud_api_library_path() {
   // A path can be configured to the location of the library.
  $path = variable_get('scorm_cloud_library_path', FALSE);
 
  if (!$path) {
    // Check if the libraries module is installed and if the example library is
    // being supplied through the libraries module.
    if (module_exists('libraries')) {
      // Check if the library is found. If no library is found libraries_get_path()
      // will still return sites/all/libraries as a path.
      $libraries = libraries_get_libraries();
      if (isset($libraries['scorm_cloud'])) {
        $path = libraries_get_path('scorm_cloud');
      }
    }
 
    // Look for the library in sites/all/libraries and sites/default/libraries
    if (file_exists('sites/default/libraries/scorm_cloud/ScormEngineService.php')) {
      return 'sites/default/libraries/scorm_cloud';
    }
    if (file_exists('sites/all/libraries/scorm_cloud/ScormEngineService.php')) {
      return 'sites/all/libraries/scorm_cloud';
    }
 
    // Check if the example library is in the include path.
    if (!$path) {
      $include_paths = explode(PATH_SEPARATOR, get_include_path());
      foreach ($include_paths as $include_path) {
        if (is_dir($include_path .'/scorm_cloud')) {
          $path = $include_path .'/scorm_cloud';
          continue;
        }
      }
    }
  }
  return $path;
}

/**
 * Return formatted values for debugging.
 */
function scorm_cloud_api_debug($value, $msg = FALSE) {
  if ($msg && function_exists('dvm')) {
    dvm($value);
  }

  // Get the results.
  ob_start();
  var_dump($call['results']);
  $results = ob_get_contents();
  ob_end_clean();
  $out = '<pre>'. $results .'</pre>';
}

/**
 * Call a method on a given service.
 *
 * This function does the actual call.
 */
function _scorm_cloud_api_call($service, $method, $params = NULL) {
  try {
    $scorm_cloud_service = scorm_cloud_api_service_engine();
    if ($scorm_cloud_service) {
      $get_service_method = 'get' . $service . 'Service';
      
      if (method_exists($scorm_cloud_service, $get_service_method)) {
        $service_object = $scorm_cloud_service->{$get_service_method}();
    
        if (method_exists($service_object, $method)) {
          $out = call_user_func_array(array($service_object, $method), (array)$params);
          $out = _scorm_cloud_api_parse_response($out, $method, $params);
          return $out;
        }
        trigger_error(t('SCORM Cloud method %method could not be found on service %service.', array('%service' => $service, '%method' => $method)), E_USER_WARNING);
        return NULL;
      }
      trigger_error(t('SCORM Cloud service %service could not be found. Requested method was %method.', array('%service' => $service, '%method' => $method)), E_USER_WARNING);
    }
  }
  catch (Exception $e) {
    trigger_error($e->getMessage(), E_USER_ERROR);
  }
  return NULL;
}

/**
 * Call a method on a given service.
 *
 * This function does the actual call.
 */
function _scorm_cloud_api_call_method($method, $params = NULL) {
  try {
    $scorm_cloud_service = scorm_cloud_api_service_engine();
    if ($scorm_cloud_service) {
      $request = $scorm_cloud_service->CreateNewRequest();
      
      if ($request) {
        $request->setMethodParams($params);
        $out = $request->CallService($method);
        $out = _scorm_cloud_api_parse_response($out, $method, $params);
        return $out;
      }
    }
  }
  catch (Exception $e) {
    trigger_error($e->getMessage(), E_USER_ERROR);
  }
}

/**
 * Parse the API response
 */
function _scorm_cloud_api_parse_response($out) {
  // Quick and dirty way of determining if the results are XML.
  if (is_string($out) && strpos($out, '<?xml version="1.0" encoding="utf-8" ?>') !== false) {
    $xml = simplexml_load_string($out);
    return $xml;
  }
  return $out;
}

